@@ -6,6 +6,9 @@
 */
 
 #include "karchivetest.h"
+
+#include "ossfuzz/karchive_fuzzer_common.h"
+
 #include <k7zip.h>
 #include <kar.h>
 #include <krcc.h>
@@ -39,9 +42,9 @@ QTEST_MAIN(KArchiveTest)
 
 static const int SIZE1 = 100;
 
-/**
+/*!
  * Writes test fileset specified archive
- * @param archive archive
+ * \a archive archive
  */
 static void writeTestFilesToArchive(KArchive *archive)
 {
@@ -160,9 +163,9 @@ static QStringList recursiveListEntries(const KArchiveDirectory *dir, const QStr
     return ret;
 }
 
-/**
+/*!
  * Verifies contents of specified archive against test fileset
- * @param archive archive
+ * \a archive archive
  */
 static void testFileData(KArchive *archive)
 {
@@ -329,7 +332,7 @@ static void testCopyTo(KArchive *archive)
     QVERIFY(QFileInfo(dirName + "executableAll").permissions() & (QFileDevice::ExeOwner | QFileDevice::ExeGroup | QFileDevice::ExeOther));
 }
 
-/**
+/*!
  * Prepares dataset for archive filter tests
  */
 void KArchiveTest::setupData()
@@ -346,6 +349,8 @@ void KArchiveTest::setupData()
 #if HAVE_XZ_SUPPORT
     QTest::newRow(".tar.lzma") << "karchivetest.tar.lzma"
                                << "application/x-lzma";
+    QTest::newRow(".tar.lz") << "karchivetest.tar.lz"
+                             << "application/x-lzip";
     QTest::newRow(".tar.xz") << "karchivetest.tar.xz"
                              << "application/x-xz";
 #endif
@@ -355,8 +360,8 @@ void KArchiveTest::setupData()
 #endif
 }
 
-/**
- * @see QTest::initTestCase()
+/*!
+ * \sa QTest::initTestCase()
  */
 void KArchiveTest::initTestCase()
 {
@@ -402,7 +407,7 @@ void KArchiveTest::testCreateTar_data()
     QTest::newRow(".tar") << "karchivetest.tar";
 }
 
-/**
+/*!
  * @dataProvider testCreateTar_data
  */
 void KArchiveTest::testCreateTar()
@@ -431,7 +436,7 @@ void KArchiveTest::testCreateTar()
     // QFile::remove(fileName);
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testCreateTarXXX()
@@ -473,7 +478,7 @@ void KArchiveTest::testCreateTarXXX()
         QVERIFY(dt.secsTo(expectedDateTime) <= 2);                                                                                                             \
     }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testReadTar() // testCreateTarGz must have been run first.
@@ -545,7 +550,7 @@ void KArchiveTest::testReadTar() // testCreateTarGz must have been run first.
     }
 }
 
-/**
+/*!
  * This tests the decompression using kfilterdev, basically.
  * To debug KTarPrivate::fillTempFile().
  *
@@ -577,7 +582,7 @@ void KArchiveTest::testUncompress()
     QVERIFY(totalSize > 26000); // 27648 here when using gunzip
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testTarFileData()
@@ -593,7 +598,7 @@ void KArchiveTest::testTarFileData()
     QVERIFY(tar.close());
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testTarCopyTo()
@@ -609,7 +614,7 @@ void KArchiveTest::testTarCopyTo()
     QVERIFY(tar.close());
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testTarReadWrite()
@@ -647,7 +652,7 @@ void KArchiveTest::testTarMaxLength_data()
     QTest::newRow("maxlength.tar.gz") << "karchivetest-maxlength.tar.gz";
 }
 
-/**
+/*!
  * @dataProvider testTarMaxLength_data
  */
 void KArchiveTest::testTarMaxLength()
@@ -1216,7 +1221,7 @@ void KArchiveTest::testZipReadRedundantDataDescriptor_data()
     QTest::newRow("withSignature") << "data/redundantDataDescriptorsWithSignature.zip";
 }
 
-/**
+/*!
  * @dataProvider testZipReadRedundantDataDescriptor_data
  */
 void KArchiveTest::testZipReadRedundantDataDescriptor()
@@ -1493,6 +1498,41 @@ void KArchiveTest::testZip64ExtraZip64Size()
     QVERIFY(zip.close());
 }
 
+void KArchiveTest::testZip64ExtraZip64SizeFirst()
+{
+    const QString fileName = QFINDTESTDATA("data/zip64_extra_zip64_size_first.zip.gz");
+    QVERIFY(!fileName.isEmpty());
+
+    KCompressionDevice gzfilter(fileName);
+    QVERIFY2(gzfilter.open(QIODevice::ReadOnly), qPrintable(gzfilter.errorString()));
+
+    // Get the actual test data, which is compressed so the repository does
+    // not have to store 4 MByte of binary data
+    auto zipData = gzfilter.read(5 * 1000 * 1000);
+    QVERIFY(zipData.startsWith("PK"));
+
+    QBuffer zipBuffer(&zipData);
+    KZip zip(&zipBuffer);
+    QEXPECT_FAIL("", "Zip64 extra at front incorrectly parsed", Abort);
+    QVERIFY2(zip.open(QIODevice::ReadOnly), qPrintable(zip.errorString()));
+
+    QCOMPARE(zip.directory()->entries().size(), 2);
+    QCOMPARE(zip.directory()->entries(), (QList{QStringLiteral("4200M.bin"), QStringLiteral("4.bin")}));
+
+    auto entry = zip.directory()->file(QStringLiteral("4200M.bin"));
+    QVERIFY(entry);
+    QCOMPARE(entry->size(), 4404019208);
+
+    auto readDev = std::unique_ptr<QIODevice>(entry->createDevice());
+    auto head = readDev->read(8);
+    QCOMPARE(head, QByteArrayLiteral("abcd\0\0\0\0"));
+    readDev->seek(entry->size() - 8);
+    auto tail = readDev->read(8);
+    QCOMPARE(tail, QByteArrayLiteral("\0\0\0\0ABCD"));
+
+    QVERIFY(zip.close());
+}
+
 void KArchiveTest::testZip64ExtraZip64Offset()
 {
     const QString fileName = QFINDTESTDATA("data/zip64_extra_zip64_localheader.oxps");
@@ -1513,6 +1553,16 @@ void KArchiveTest::testZip64ExtraZip64Offset()
     QVERIFY(zip.close());
 }
 
+void KArchiveTest::testZipOssFuzzIssue433303801()
+{
+    const QString fileName = QFINDTESTDATA("data/ossfuzz_issue_433303801.zip");
+    QVERIFY(!fileName.isEmpty());
+
+    KZip zip(fileName);
+    QVERIFY(zip.open(QIODevice::ReadOnly));
+    traverseArchive(zip.directory());
+}
+
 void KArchiveTest::testRcc()
 {
     const QString rccFile = QFINDTESTDATA("data/runtime_resource.rcc"); // was copied from qtbase/tests/auto/corelib/io/qresourceengine
@@ -1565,8 +1615,8 @@ void KArchiveTest::testAr()
     QVERIFY(ar.close());
 }
 
-/**
- * @see QTest::cleanupTestCase()
+/*!
+ * \sa QTest::cleanupTestCase()
  */
 void KArchiveTest::cleanupTestCase()
 {
@@ -1582,7 +1632,7 @@ void KArchiveTest::cleanupTestCase()
 
 #if HAVE_XZ_SUPPORT
 
-/**
+/*
  * Prepares dataset for 7zip tests
  */
 void KArchiveTest::setup7ZipData()
@@ -1591,7 +1641,7 @@ void KArchiveTest::setup7ZipData()
     QTest::newRow(".7z") << "karchivetest.7z";
 }
 
-/**
+/*
  * @dataProvider testCreate7Zip_data
  */
 void KArchiveTest::testCreate7Zip()
@@ -1614,7 +1664,7 @@ void KArchiveTest::testCreate7Zip()
     }
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::testRead7Zip() // testCreate7Zip must have been run first.
@@ -1663,7 +1713,7 @@ void KArchiveTest::testRead7Zip() // testCreate7Zip must have been run first.
     }
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::test7ZipFileData()
@@ -1679,7 +1729,7 @@ void KArchiveTest::test7ZipFileData()
     QVERIFY(k7zip.close());
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::test7ZipCopyTo()
@@ -1695,7 +1745,7 @@ void KArchiveTest::test7ZipCopyTo()
     QVERIFY(k7zip.close());
 }
 
-/**
+/*!
  * @dataProvider setupData
  */
 void KArchiveTest::test7ZipReadWrite()
@@ -1727,7 +1777,7 @@ void KArchiveTest::test7ZipReadWrite()
     QFile::remove(fileName);
 }
 
-/**
+/*!
  * @dataProvider test7ZipMaxLength_data
  */
 void KArchiveTest::test7ZipMaxLength()
@@ -1836,6 +1886,43 @@ void KArchiveTest::test7ZipReadNumber()
     QVERIFY(k7zip.close());
 }
 
+void KArchiveTest::test7ZipFileNameEndsInSlash()
+{
+    const QString fileName = QFINDTESTDATA("data/filename_ends_in_slash.7z");
+    QVERIFY(!fileName.isEmpty());
+
+    K7Zip k7zip(fileName);
+    QVERIFY(!k7zip.open(QIODevice::ReadOnly));
+}
+
+void KArchiveTest::test7ZipOssFuzzIssues_data()
+{
+    QTest::addColumn<QString>("fileName");
+    QTest::newRow("issue_440829292") << "data/ossfuzz_issue_440829292.7z";
+    QTest::newRow("issue_441906077") << "data/ossfuzz_issue_441906077.7z";
+    QTest::newRow("testcase_4566647131406336") << "data/ossfuzz_testcase_4566647131406336.7z";
+    QTest::newRow("testcase_5560695602348032") << "data/ossfuzz_testcase_5560695602348032.7z";
+    QTest::newRow("testcase_5581009572921344") << "data/ossfuzz_testcase_5581009572921344.7z";
+    QTest::newRow("testcase_5821682553257984") << "data/ossfuzz_testcase_5821682553257984.7z";
+    QTest::newRow("testcase_6077171694370816") << "data/ossfuzz_testcase_6077171694370816.7z";
+    QTest::newRow("testcase_6096742417498112") << "data/ossfuzz_testcase_6096742417498112.7z";
+    QTest::newRow("testcase_6593198198947840") << "data/ossfuzz_testcase_6593198198947840.7z";
+    QTest::newRow("testcase_6653406484955136") << "data/ossfuzz_testcase_6653406484955136.7z";
+}
+
+void KArchiveTest::test7ZipOssFuzzIssues()
+{
+    QFETCH(QString, fileName);
+
+    const QString filePath = QFINDTESTDATA(fileName);
+    QVERIFY(!filePath.isEmpty());
+
+    K7Zip k7zip(filePath);
+    if (k7zip.open(QIODevice::ReadOnly)) {
+        k7zip.close();
+    }
+}
+
 #if HAVE_OPENSSL_SUPPORT
 void KArchiveTest::test7ZipPasswordProtected()
 {
