@@ -587,7 +587,8 @@ void K7Zip::setPassword(const QString &password) {
 bool K7Zip::passwordNeeded() const
 {
     for (int i = 0; i < d->folders.size(); i++) {
-        if (d->folders.at(i)->isEncrypted()) {
+        const Folder *folder = d->folders.at(i);
+        if (folder && folder->isEncrypted()) {
             return true;
         }
     }
@@ -947,7 +948,7 @@ bool K7Zip::K7ZipPrivate::readUnpackInfo()
         return false;
     }
 
-    int numFolders = readNumber();
+    const int numFolders = readNumber();
     qDeleteAll(folders);
     folders.clear();
     int external = readByte();
@@ -976,6 +977,10 @@ bool K7Zip::K7ZipPrivate::readUnpackInfo()
         return false;
     }
 
+    if (numFolders != folders.count()) {
+        return false;
+    }
+
     for (int i = 0; i < numFolders; ++i) {
         Folder *folder = folders.at(i);
         if (!folder) {
@@ -1002,6 +1007,9 @@ bool K7Zip::K7ZipPrivate::readUnpackInfo()
             readHashDigests(numFolders, crcsDefined, crcs);
             for (int i = 0; i < numFolders; i++) {
                 Folder *folder = folders.at(i);
+                if (!folder) {
+                    continue;
+                }
                 folder->unpackCRCDefined = crcsDefined[i];
                 folder->unpackCRC = crcs[i];
             }
@@ -1057,6 +1065,9 @@ bool K7Zip::K7ZipPrivate::readSubStreamsInfo()
                 sum += size;
             }
         }
+        if (i >= folders.size() || !folders.at(i)) {
+            return false;
+        }
         unpackSizes.append(folders.at(i)->getUnpackSize() - sum);
     }
 
@@ -1777,8 +1788,8 @@ QByteArray K7Zip::K7ZipPrivate::readAndDecodePackedStreams(bool readMainStreamIn
         seqInStreams.reserve(mainCoder->numInStreams);
         coderIndexes.reserve(mainCoder->numInStreams);
         for (int j = 0; j < (int)mainCoder->numInStreams; j++) {
-            int seqInStream;
-            quint32 coderIndex;
+            int seqInStream = 0;
+            quint32 coderIndex = 0;
             getInStream(folder, startInIndex + j, seqInStream, coderIndex);
             seqInStreams.append(seqInStream);
             coderIndexes.append(coderIndex);
@@ -1794,6 +1805,9 @@ QByteArray K7Zip::K7ZipPrivate::readAndDecodePackedStreams(bool readMainStreamIn
 
         QList<QByteArray> datas;
         for (int j = 0; j < (int)mainCoder->numInStreams; j++) {
+            if (j+i >= packSizes.size()) {
+                return QByteArray();
+            }
             quint64 size = packSizes[j + i];
             std::unique_ptr<char[]> encodedBuffer(new char[size]);
             QIODevice *dev = q->device();
@@ -2659,7 +2673,21 @@ bool K7Zip::openArchive(QIODevice::OpenMode mode)
     }
 
     // read files info
-    int numFiles = d->readNumber();
+    const quint64 fileNumFiles = d->readNumber();
+    if (fileNumFiles > std::numeric_limits<qsizetype>::max()) {
+        // We iterate over QList with at() and at() accepts qsizetype as input parameter
+        setErrorString(tr("Archive has %1 files which is more than the supported amount (%2)").arg(fileNumFiles).arg(std::numeric_limits<qsizetype>::max()));
+        return false;
+    }
+
+    // TODO Explore a way to be able to support more files.
+    // Right now we limit to 1000 million files
+    static const qsizetype MAX_FILE_NUMBER = 1000 * 1000 * 1000;
+    const qsizetype numFiles = fileNumFiles;
+    if (numFiles > MAX_FILE_NUMBER) {
+        setErrorString(tr("Archive has %1 files which is more than the supported amount (%2)").arg(numFiles).arg(MAX_FILE_NUMBER));
+        return false;
+    }
     for (int i = 0; i < numFiles; ++i) {
         d->fileInfos.append(new FileInfo);
     }
@@ -2814,7 +2842,8 @@ bool K7Zip::openArchive(QIODevice::OpenMode mode)
         emptyFileVector.fill(false, numEmptyStreams);
     }
 
-    for (int i = 0; i < numEmptyStreams; i++) {
+    const int minNumEmptyStreamsAndAntiFile = std::min(numEmptyStreams, static_cast<int>(antiFileVector.count()));
+    for (int i = 0; i < minNumEmptyStreamsAndAntiFile; i++) {
         if (antiFileVector[i]) {
             numAntiItems++;
         }
@@ -2850,6 +2879,10 @@ bool K7Zip::openArchive(QIODevice::OpenMode mode)
         if (fileInfo->hasStream) {
             fileInfo->isDir = false;
             isAnti = false;
+            if (sizeIndex >= d->unpackSizes.size() || sizeIndex >= d->digests.size()) {
+                return false;
+            }
+
             fileInfo->size = d->unpackSizes[sizeIndex];
             fileInfo->crc = d->digests[sizeIndex];
             fileInfo->crcDefined = d->digestsDefined[sizeIndex];
@@ -2894,7 +2927,10 @@ bool K7Zip::openArchive(QIODevice::OpenMode mode)
         } else {
             entryName = fileInfo->path.mid(index + 1);
         }
-        Q_ASSERT(!entryName.isEmpty());
+        if (entryName.isEmpty()) {
+            // fileInfo->path ends in /
+            return false;
+        }
 
         QDateTime mTime;
         if (d->mTimesDefined.size() > i && d->mTimesDefined[i]) {
