@@ -31,6 +31,23 @@
 
 static const int max_path_len = 4095; // maximum number of character a path may contain
 
+static quint16 parseUi16(const char *buffer)
+{
+    return quint16((uchar)buffer[0] | (uchar)buffer[1] << 8);
+}
+
+static uint parseUi32(const char *buffer)
+{
+    return uint((uchar)buffer[0] | (uchar)buffer[1] << 8 | (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
+}
+
+static quint64 parseUi64(const char *buffer)
+{
+    const uint a = uint((uchar)buffer[0] | (uchar)buffer[1] << 8 | (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
+    const uint b = uint((uchar)buffer[4] | (uchar)buffer[5] << 8 | (uchar)buffer[6] << 16 | (uchar)buffer[7] << 24);
+    return (a | (quint64)b << 32);
+}
+
 static void transformToMsDos(const QDateTime &_dt, char *buffer)
 {
     const QDateTime dt = _dt.isValid() ? _dt : QDateTime::currentDateTime();
@@ -55,13 +72,13 @@ static void transformToMsDos(const QDateTime &_dt, char *buffer)
 
 static uint transformFromMsDos(const char *buffer)
 {
-    quint16 time = (uchar)buffer[0] | ((uchar)buffer[1] << 8);
+    const quint16 time = parseUi16(buffer);
     int h = time >> 11;
     int m = (time & 0x7ff) >> 5;
     int s = (time & 0x1f) * 2;
     QTime qt(h, m, s);
 
-    quint16 date = (uchar)buffer[2] | ((uchar)buffer[3] << 8);
+    const quint16 date = parseUi16(buffer + 2);
     int y = (date >> 9) + 1980;
     int o = (date & 0x1ff) >> 5;
     int d = (date & 0x1f);
@@ -71,18 +88,6 @@ static uint transformFromMsDos(const char *buffer)
     return dt.toSecsSinceEpoch();
 }
 
-static uint parseUi32(const char *buffer)
-{
-    return uint((uchar)buffer[0] | (uchar)buffer[1] << 8 | (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
-}
-
-static quint64 parseUi64(const char *buffer)
-{
-    const uint a = uint((uchar)buffer[0] | (uchar)buffer[1] << 8 | (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
-    const uint b = uint((uchar)buffer[4] | (uchar)buffer[5] << 8 | (uchar)buffer[6] << 16 | (uchar)buffer[7] << 24);
-    return (a | (quint64)b << 32);
-}
-
 // == parsing routines for zip headers
 
 /** all relevant information about parsing file information */
@@ -145,7 +150,7 @@ static bool parseExtTimestamp(const char *buffer, int size, bool islocal, ParseF
             // qCDebug(KArchiveLog) << "premature end of extended timestamp (#2)";
             return false;
         } /*end if*/
-        pfi.mtime = uint((uchar)buffer[0] | (uchar)buffer[1] << 8 | (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
+        pfi.mtime = parseUi32(buffer);
         buffer += 4;
         size -= 4;
     } /*end if*/
@@ -161,7 +166,7 @@ static bool parseExtTimestamp(const char *buffer, int size, bool islocal, ParseF
             // qCDebug(KArchiveLog) << "premature end of extended timestamp (#3)";
             return true;
         } /*end if*/
-        pfi.atime = uint((uchar)buffer[0] | (uchar)buffer[1] << 8 | (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
+        pfi.atime = parseUi32(buffer);
         buffer += 4;
         size -= 4;
     } /*end if*/
@@ -171,7 +176,7 @@ static bool parseExtTimestamp(const char *buffer, int size, bool islocal, ParseF
             // qCDebug(KArchiveLog) << "premature end of extended timestamp (#4)";
             return true;
         } /*end if*/
-        pfi.ctime = uint((uchar)buffer[0] | (uchar)buffer[1] << 8 | (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
+        pfi.ctime = parseUi32(buffer);
         buffer += 4;
     } /*end if*/
 
@@ -199,15 +204,11 @@ static bool parseInfoZipUnixOld(const char *buffer, int size, bool islocal, Pars
         return false;
     }
 
-    pfi.atime = uint((uchar)buffer[0] | (uchar)buffer[1] << 8 | (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
-    buffer += 4;
-    pfi.mtime = uint((uchar)buffer[0] | (uchar)buffer[1] << 8 | (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
-    buffer += 4;
+    pfi.atime = parseUi32(buffer);
+    pfi.mtime = parseUi32(buffer + 4);
     if (islocal && size >= 12) {
-        pfi.uid = (uchar)buffer[0] | (uchar)buffer[1] << 8;
-        buffer += 2;
-        pfi.gid = (uchar)buffer[0] | (uchar)buffer[1] << 8;
-        buffer += 2;
+        pfi.uid = parseUi16(buffer + 8);
+        pfi.gid = parseUi16(buffer + 10);
     } /*end if*/
     return true;
 }
@@ -254,10 +255,9 @@ static bool parseInfoZipUnixNew(const char *buffer, int size, bool islocal,
 static bool parseExtraField(const char *buffer, int size, ParseFileInfo &pfi)
 {
     while (size >= 4) { // as long as a potential extra field can be read
-        int magic = (uchar)buffer[0] | (uchar)buffer[1] << 8;
-        buffer += 2;
-        int fieldsize = (uchar)buffer[0] | (uchar)buffer[1] << 8;
-        buffer += 2;
+        int magic = parseUi16(buffer);
+        int fieldsize = parseUi16(buffer + 2);
+        buffer += 4;
         size -= 4;
 
         if (fieldsize > size) {
@@ -355,14 +355,16 @@ static bool seekPostDataDescriptor(QIODevice *dev, bool isZip64)
     const qsizetype descriptorSize = isZip64 ? 24 : 16;
 
     QByteArray header;
-    auto oldPos = dev->pos();
+    const qint64 oldPos = dev->pos();
 
     while (seekAnyHeader(dev, header, descriptorSize)) {
         // qCDebug(KArchiveLog) << "Possible data descriptor header at" << dev->pos() << header;
         if (header.startsWith("PK\x07\x08")) {
-            qint64 compressedSize = isZip64 ? parseUi64(header.constData() + 8) //
-                                            : parseUi32(header.constData() + 8);
-            if (oldPos + compressedSize == dev->pos()) {
+            const qint64 compressedSize = isZip64 ? parseUi64(header.constData() + 8) //
+                                                  : parseUi32(header.constData() + 8);
+            // Do not move compressedSize to the left side of the check
+            // we have on the left on purpose to prevent addition overflows
+            if ((compressedSize > 0) && (oldPos == dev->pos() - compressedSize)) {
                 dev->seek(dev->pos() + descriptorSize);
                 return true;
             }
@@ -507,17 +509,17 @@ bool KZip::openArchive(QIODevice::OpenMode mode)
                 setErrorString(tr("Invalid ZIP file. Unexpected end of file. (Error code: %1)").arg(4));
                 return false;
             }
-            int neededVersion = (uchar)buffer[0] | (uchar)buffer[1] << 8;
+            int neededVersion = parseUi16(buffer);
             bool isZip64 = neededVersion >= 45;
 
             int gpf = (uchar)buffer[2]; // "general purpose flag" not "general protection fault" ;-)
-            int compression_mode = (uchar)buffer[4] | (uchar)buffer[5] << 8;
+            int compression_mode = parseUi16(buffer + 4);
             uint mtime = transformFromMsDos(buffer + 6);
 
             const qint64 compr_size = parseUi32(buffer + 14);
             const qint64 uncomp_size = parseUi32(buffer + 18);
-            const int namelen = uint(uchar(buffer[22])) | uint(uchar(buffer[23])) << 8;
-            const int extralen = uint(uchar(buffer[24])) | uint(uchar(buffer[25])) << 8;
+            const int namelen = parseUi16(buffer + 22);
+            const int extralen = parseUi16(buffer + 24);
 
             /*
               qCDebug(KArchiveLog) << "general purpose bit flag: " << gpf;
@@ -656,15 +658,15 @@ bool KZip::openArchive(QIODevice::OpenMode mode)
             }
 
             // length of extra attributes
-            int extralen = (uchar)buffer[31] << 8 | (uchar)buffer[30];
+            const int extralen = parseUi16(buffer + 30);
             // length of comment for this file
-            int commlen = (uchar)buffer[33] << 8 | (uchar)buffer[32];
+            const int commlen = parseUi16(buffer + 32);
             // compression method of this file
-            int cmethod = (uchar)buffer[11] << 8 | (uchar)buffer[10];
-            // int gpf = (uchar)buffer[9] << 8 | (uchar)buffer[10];
+            const int cmethod = parseUi16(buffer + 10);
+            // int gpf =  parseUi16(buffer + 8);
             // qCDebug(KArchiveLog) << "general purpose flag=" << gpf;
             // length of the fileName (well, pathname indeed)
-            int namelen = (uchar)buffer[29] << 8 | (uchar)buffer[28];
+            const int namelen = parseUi16(buffer + 28);
             if (namelen <= 0) {
                 setErrorString(tr("Invalid ZIP file, file path name length is zero"));
                 return false;
@@ -735,7 +737,7 @@ bool KZip::openArchive(QIODevice::OpenMode mode)
             int access = 0100644;
 
             if (os_madeby == 3) { // good ole unix
-                access = (uchar)buffer[40] | (uchar)buffer[41] << 8;
+                access = parseUi16(buffer + 40);
             }
 
             QString entryName;
@@ -1471,6 +1473,12 @@ QByteArray KZipFileEntry::data() const
     QIODevice *dev = createDevice();
     QByteArray arr;
     if (dev) {
+        const qint64 devSize = dev->size();
+        if (devSize > kMaxQByteArraySize) {
+            qCWarning(KArchiveLog) << "KZipFileEntry::data: Failed to allocate memory for file of size" << devSize;
+            delete dev;
+            return {};
+        }
         arr = dev->readAll();
         delete dev;
     }
@@ -1481,14 +1489,14 @@ QIODevice *KZipFileEntry::createDevice() const
 {
     // qCDebug(KArchiveLog) << "creating iodevice limited to pos=" << position() << ", csize=" << compressedSize();
     // Limit the reading to the appropriate part of the underlying device (e.g. file)
-    KLimitedIODevice *limitedDev = new KLimitedIODevice(archive()->device(), position(), compressedSize());
+    std::unique_ptr limitedDev = std::make_unique<KLimitedIODevice>(archive()->device(), position(), compressedSize());
     if (encoding() == 0 || compressedSize() == 0) { // no compression (or even no data)
-        return limitedDev;
+        return limitedDev.release();
     }
 
     if (encoding() == 8) {
         // On top of that, create a device that uncompresses the zlib data
-        KCompressionDevice *filterDev = new KCompressionDevice(limitedDev, true, KCompressionDevice::GZip);
+        KCompressionDevice *filterDev = new KCompressionDevice(std::move(limitedDev), KCompressionDevice::GZip, size());
 
         if (!filterDev) {
             return nullptr; // ouch
@@ -1502,6 +1510,5 @@ QIODevice *KZipFileEntry::createDevice() const
 
     qCCritical(KArchiveLog) << "This zip file contains files compressed with method" << encoding() << ", this method is currently not supported by KZip,"
                             << "please use a command-line tool to handle this file.";
-    delete limitedDev;
     return nullptr;
 }
